import { Component, ViewChild, ElementRef, AfterViewInit, Inject } from '@angular/core';
import { chart } from 'highcharts';
import { Highcharts } from 'angular-highcharts';

import { GraphElementComponent } from './graph-element/graph-element.component';

import { Report, ReportService } from './../report.service';
import { Observable } from 'rxjs/Observable';

@Component({
  selector: 'graphs-page',
  templateUrl: './graphs-page.component.html',
  styleUrls: ['./graphs-page.scss']
})

export class GraphsPageComponent implements AfterViewInit {
  @ViewChild(GraphElementComponent)
  private graphComponent: GraphElementComponent;

  report: Report;
  sus_arr: Array<number>;
  sus_total: number;
  non_sus_arr: Array<number>;
  non_sus_total: number;

  graphOptions: any;
  selectedGraphType: string;
  selectedInterval: string;
  intervalList: Array<string>;
  startDate: Date;
  endDate: Date;
  graphStartDate: Date;

  constructor(@Inject('ReportService') private reportService: ReportService) {

    this.intervalList = ["Mensal", "Diário" ];
    this.selectedInterval = this.intervalList[1];

    this.endDate = new Date();
    this.startDate = new Date();
    this.startDate.setMonth(this.endDate.getMonth() - 1);
    this.graphStartDate = new Date();

    this.sus_arr = [];
    this.sus_total = 0;
    this.non_sus_arr = [];
    this.non_sus_total = 0;

    this.setChartOptions("line");
    this.updateChart();
  }
  
  ngAfterViewInit() {
  }

  private setChartOptions(graphType: string) {
    this.selectedGraphType = graphType;
    /* We need to reset the options to make sure previous graphs don't mess with the current */
    this.graphOptions = {}; 

    /* Common options */
    this.graphOptions.colors = ['#8e8cc4', '#4f425b', '#c5dcf2'];
    this.graphOptions.subtitle =  { text: "Unidade monitorada: Hospital Erasto Gaertner" };
    this.graphOptions.credits = { enabled: false };

    /* Populate graph based on its type */
    if(this.selectedGraphType == "line") {

      let interval: number = 24 * 3600 * 1000; // Graph tick interval in milliseconds
      if(this.selectedInterval == "Mensal" || this.selectedInterval == "Anual")
        interval *= 30;
      if(this.selectedInterval == "Anual")
        interval *= 12;

      this.graphOptions.title = { text: "Distribuição dos atendimentos ao longo do tempo" };
      this.graphOptions.yAxis = { title: { text: "Número de pacientes"} };
      this.graphOptions.xAxis = {
        type: "datetime",
        tickInterval: interval
      };

      this.graphOptions.series = [{
        name: "Geral",
        data: this.non_sus_arr,
        pointStart: Date.UTC(this.graphStartDate.getFullYear(), this.graphStartDate.getMonth(), this.graphStartDate.getDay()),
        pointInterval: interval
      }, {
        name: "SUS",
        data: this.sus_arr,
        pointStart: Date.UTC(this.graphStartDate.getFullYear(), this.graphStartDate.getMonth(), this.graphStartDate.getDay()),
        pointInterval: interval
      }
    ];

    } else if (this.selectedGraphType == "pie") {

      this.graphOptions.chart = { type: 'pie' };
      this.graphOptions.title = { text: 'Proporção de atendidos pelo SUS' };
      this.graphOptions.tooltip = { pointFormat: '{series.name}: <b>{point.percentage:.1f}%</b>' }
      this.graphOptions.plotOptions = {
        pie: {
          dataLabels: {
          enabled: true,
          format: '<b>{point.name}</b>: {point.y}'
          }
        }
      }
      this.graphOptions.series = [{
        name: 'Percentual de Pacientes',
        data: [{
          name: 'Geral',
          y: this.non_sus_total
        },{
          name: 'SUS',
          y: this.sus_total
        }]
      }]

    }

  }


  setTypeofGraph(newType: string) {
    this.selectedGraphType = newType;
  }

  private updateChart() {
    /* Retrieve graph data from service */
    this.reportService.getReports(this.selectedInterval, this.startDate, this.endDate)
        .subscribe( report => {
            this.report = report;
            this.sus_arr = [];
            this.sus_total = 0;
            this.non_sus_arr = [];
            this.non_sus_total = 0;
            this.graphStartDate = new Date(this.report.sus[0].date);
          
            for(let iter of this.report.sus) {
              this.sus_arr.push(iter.uses);
              this.sus_total += iter.uses;
            }         
    
            for(let iter of this.report.not_sus) {
              this.non_sus_arr.push(iter.uses);
              this.non_sus_total += iter.uses;
            }
                  
        },
        error => console.log(error),
        () => {
          this.setChartOptions(this.selectedGraphType);
          this.graphComponent.reset(this.graphOptions);
        }
      );
  }
  
}