import {Component, OnInit, ViewChild} from '@angular/core';
import { HospitalSelectorComponent } from '../hospital-selector/hospital-selector.component';
import { DataApiService } from '../data-api.service';

import * as ol from 'openlayers';
import * as $ from 'jquery';

declare function require(url: string);
declare var mapcomp: any;


@Component({
	selector: 'app-maps',
	templateUrl: './maps-page.component.html',
	styleUrls: ['./maps-page.component.scss']
})


export class MapsPageComponent implements OnInit {
	private dataApi: DataApiService;
	
	// OpenLayer Map attributes
	private map;
	private source;
	private layers = [];
	private view;
	private mainVectorSource;
	private mainLayer;
	private overlay;
	private popupContainer;
	
	// OpenLayers features cache (used in breadcrumb)
	private stateSelected;
	private countieSelected;
	
	
	// Controller used in clickables features handlers on map
	private stageSelection : string = "init";
	
	
	// List of agents (loaded in init in ngOnInit) 
	private agents;
	
	
	// Style of feature without agent (normal and selected)
	private layerStyle = [
		new ol.style.Style({
			stroke: new ol.style.Stroke({
				color: 'blue',
				width: 1
			}),
			fill: new ol.style.Fill({
				color: 'rgba(0, 0, 255, 0.1)'
			})
		}),
		new ol.style.Style({
			stroke: new ol.style.Stroke({
				color: 'black',
				width: 1
			}),
			fill: new ol.style.Fill({
				color: 'rgba(0, 0, 255, 0.1)'
			})
		})
	];
	
	// Style of feature with agent (normal and selected)
	private markedLayerStyle = [
			new ol.style.Style({
				stroke: new ol.style.Stroke({
					color: 'blue',
					width: 1
				}),
				fill: new ol.style.Fill({
					color: 'rgba(0, 255, 0, 0.2)'
				})
			}),
			new ol.style.Style({
				stroke: new ol.style.Stroke({
					color: 'red',
					width: 1
				}),
				fill: new ol.style.Fill({
					color: 'rgba(0, 255, 0, 0.2)'
				})
			})
		];
	
	constructor(private da: DataApiService) {
		this.dataApi = da;
	}
	
	
	public selectCountry(){
		this.stageSelection = "init";
		this.mainLayer.getSource().clear();
		$(this.popupContainer).css("display",'none');
		$("#countieSelected").html("");
		$("#stateSelected").html("");
		
		var mapcomp = this;
		this.dataApi.loadStates(function(result){
			mapcomp.reloadMap(result, [-50,-20], 4);
		});
	}
	
	public selectFeatureState(){
		this.stageSelection = "init";
		this.selectFeature(this.stateSelected);
		
	}
	public selectFeatureCountie(){
		this.stageSelection = "state";
		this.selectFeature(this.countieSelected);
	}
	
	// Called only on ngOnInit
	private initMap(features, center, zoom){
		this.mainLayer = this.createLayer([],this.layerStyle, {global: true});
		var map = new ol.Map({
			layers: [
				new ol.layer.Tile({source: new ol.source.OSM()}),
				this.mainLayer
			],
			target: 'map',
			view: new ol.View({
				projection: 'EPSG:4326',
				center: center,
				zoom: zoom,
			})
		});
		this.map = map;
		
		let mapComponent = this;
		map.on('singleclick', function(evt) {
			var feature = map.forEachFeatureAtPixel(evt.pixel, function(feature, layer) {
				return feature;
			});
			if (feature) {
				mapComponent.selectFeature(feature);
			}
		});
	}
	
	private resetLayers(){
		for(var i in this.layers){
			this.map.removeLayer(this.layers[i]);
		}
		this.layers = [];
	}
	
	// Create a layer that contain same features
	// the options can be used to set that layer to not 
	// be reseted on resetLayers()
	private createLayer(features, styles, options={}){
		var mapcomp = this;
		features = features.map(function(x){
			var copieProp = $.extend({}, x["N"]);
			x = new ol.Feature(x["N"]["geometry"]);
			x["N"] = copieProp;
			return x;
		});
		
		var source = new ol.source.Vector({
			features: features
		});
		
		var layer = new ol.layer.Vector({
			source: source,
			style: styles
		});
		
		if(options){
			if(!options["global"]){
				this.layers.push(layer);
			}
		}
		return layer;
	}
	
	// Reset all layers, add new features and focus on some point
	// The features gain a handler that highlight it when it is selected
    // The parameter features must be a FeatureCollection
	private reloadMap(features, center, zoom){
		
		var mapcomp = this;
		this.resetLayers();
		var geojsonObject = {
			'type': 'FeatureCollection',
			'features': features
		};
        var cp = features;

		features = (new ol.format.GeoJSON()).readFeatures(geojsonObject);
        
        for(var i in features){
            var newpoly = new ol.geom.Polygon([cp[i].geometry.coordinates]);
            features[i].setGeometry(newpoly);
        }
        
        
		var filter;
		var filter2;
		if(this.stageSelection == "init"){
			filter = "state";
			filter2 = "Name";
		}
		if(this.stageSelection == "state"){
			filter = "countie";
			filter2 = "Name";
		}
		
		var id_hospitals = [];
		var arrayToMark = mapcomp.agents.map(function(x){
			return x[filter];
		});
		
        // Find the features that contains agents
		var markedLayerFeatures = [];
		for(var i in features){
			if(arrayToMark.indexOf(features[i]["N"][filter2]) != -1){
				var f = features.splice(i,1)[0];
				var hid = features[i]["N"]["Name"];
				markedLayerFeatures.push(f);
				if("id_hospitals" in f)
					f.id_hospitals.push(hid);
				else
					f.id_hospitals = [hid];
			}
		}
		
		var layer = this.createLayer(features,this.layerStyle);
		var layer2 = this.createLayer(markedLayerFeatures,this.markedLayerStyle);
		
		this.map.addLayer(layer);
		this.map.addLayer(layer2);
		
		
		var hoverInteraction = new ol.interaction.Select({
			condition: ol.events.condition.pointerMove,
			layers:[layer,layer2]  //Setting layers to be hovered
		});
		this.map.addInteraction(hoverInteraction);
		
		this.map.getView().fit(new ol.geom.Point(center), {size:this.map.getSize(), maxZoom: zoom, duration: 1000});
		
	}
	
	//Handler of click or select a feature of the map
	private selectFeature(feature, callback=null){
		
        var extent = feature["N"].geometry.getExtent();
		var mapcomp = this;

		//Trigger when the zoom is complete
		var afterZoom = function(){
			mapcomp.dataApi.selectCountiesFromState(feature["N"]["Name"] , function(result){
				mapcomp.reloadMap(
					result,
					mapcomp.map.getView().getCenter(),
					mapcomp.map.getView().getZoom()
				);
			});
		}
		
		//Only need reload map when the focus is on country
		if(this.stageSelection != "init"){
			afterZoom = function(){};
		}
		
		//Remove all the markers points
		if(this.stageSelection != "hospital" && this.stageSelection != "countie"){
			this.mainLayer.getSource().clear();
		}
		
		//Hide the popup
		if(this.stageSelection != "hospital"){
			$(this.popupContainer).css("display",'none');
		}
		
		
		switch(this.stageSelection){
			case "init":
				this.stageSelection = "state";
				this.stateSelected = feature;
				$("#stateSelected").html(feature["N"]["Name"]);
				$("#countieSelected").html("");
				break;
			case "state":
				this.stageSelection = "countie";
				this.selectCountie(feature["N"]["Name"]);
				this.countieSelected = feature;
				$("#countieSelected").html(feature["N"]["Name"]);
				break;
			case "countie":
				this.stageSelection = "hospital";
				this.zoomToHospital(feature["N"]["data"]["id"]);
				var hospital = this.dataApi.getAgentById(feature["N"]["data"]["id"], this.agents);
				break;
			
		}
		
		//Pan to focus the feature selected
		this.map.getView().fit(extent, {size:this.map.getSize(), maxZoom: 17, duration: 1000, callback: function(){
			afterZoom();
		}});
	}
	
	private selectCountie(nameCountie){
		var mapcomp = this;
		mapcomp.resetLayers();
		this.dataApi.loadAgents(function(result){
			for(var i in result){
				var coordinates = {
					lat: result[i].geo_lat,
					lon: result[i].geo_long
				};
				mapcomp.addMarkers(coordinates,result[i]);
			}
		}, nameCountie);
	}
	
	// Pan to same agent and show a popup on the map with this name
	private zoomToHospital(hospital_id){
		var hospital = this.dataApi.getAgentById(hospital_id, this.agents);	
		if(hospital == null)
			return;

		this.selectCountie(hospital.countie);
		this.overlay = new ol.Overlay({
			element: this.popupContainer,
			positioning: 'bottom-center',
			offset: [0, 0]
		});
		var pos : [number, number] = [hospital.geo_long, hospital.geo_lat] ;
		var point = new ol.geom.Point(pos);
		this.reloadMap([],pos,17);
		this.map.addOverlay(this.overlay);
		
		this.popupContainer.innerHTML = hospital.hospital_name;
		this.overlay.setPosition(pos);
		$(this.popupContainer).css("display",'inline-block');
	}
	
	// Create a marker on some coordinate in the map and add the data
	// that represent an agent object that will be used to the handler
	// click on it
	private addMarkers(coordinate, data): void {
		var lon = coordinate.lon;
		var lat = coordinate.lat;
		var iconStyle = new ol.style.Style({
			image: new ol.style.Icon(({
				anchor: [0.5, 46],
				anchorXUnits: 'fraction',
				anchorYUnits: 'pixels',
				opacity: 0.75,
				src: '/assets/images/marker.png'
			}))
		});

		var pos : [number, number] = [lon, lat] ;
		var point = new ol.geom.Point(pos);
		var iconFeature = new ol.Feature({
			geometry: point,
			data: data
		});
		iconFeature.setStyle(iconStyle);
		this.mainLayer.getSource().addFeature(iconFeature);
	}

	
	
	//Communicate with selector component
	private selectAgent = function(agent, mapcomp){
		
		var features;
		mapcomp.dataApi.selectStateFeature(agent.state,function(x){
			var geojsonObject = {
				'type': 'FeatureCollection',
				'features': [x.features[0]]
			};
			features = (new ol.format.GeoJSON()).readFeatures(geojsonObject);
			mapcomp.stateSelected = features[0];
			$("#stateSelected").html(agent.state);
		});
		
		mapcomp.dataApi.selectCountiesFromState(agent.state,function(x){
			var features;
			for(var y in x.features){
				if(x.features[y].properties["Name"] == agent.countie)
					features = [x.features[y]];
			}
			var geojsonObject = {
				'type': 'FeatureCollection',
				'features': features
			};
			features = (new ol.format.GeoJSON()).readFeatures(geojsonObject);
			mapcomp.countieSelected = features[0];
			$("#countieSelected").html(agent.countie);
		});
		
		mapcomp.stageSelection = "hospital";
		mapcomp.zoomToHospital(agent.id);
		
	}
	
	@ViewChild(HospitalSelectorComponent)
		set pane(v: HospitalSelectorComponent) {
		setTimeout(() => {
			v.itemTriggerClick = this.selectAgent; //On click on agent
			v.father = this;
		}, 0);
	}
	//
	
	ngOnInit() {
		var mapcomp = this;
		this.initMap([], [-50,-20], 4);
		this.popupContainer = document.getElementById('popup-container');
		$(this.popupContainer).css("display",'none');
		mapcomp.dataApi.loadAgents(function(result){
			mapcomp.agents = result;
			mapcomp.selectCountry();
			mapcomp.map.updateSize();
		});
	}
}
